# a function to produce simulation data
# here we consider: the number of CpG sites in a given region, the number of reads for each CpG site of each of cells, the number of cells and the rate of missing data.
# the model parameters include: lambda, theta1 and theta2
simul.data <- function(cpg.num=10, read.num=10, ncells=20, missing.rate=0.2, lambda=0.4, theta1=0.3, theta2=0.8) {
  # initialization of variables
  count <- c(1); reads <- c(); 
  
  # simulation includes:
  # (1) the 'lambda' proportion of cells have a methylation probability theta1 and  the '1-lambda' proportion of cells have the methylation probability theta2;
  # (2) the read counts for each CpG site follows Poisson distribution by a mean value of 'read.num';
  # (3) the methylated counts for each CpG site follow Binomial distribution.
  for(i in 1:ncells) if(runif(1) >= missing.rate) if(i/ncells <= lambda) {
    for(c in 1:cpg.num){
      if(runif(1) >= missing.rate) {
        cg.r.num <- rpois(1,read.num)+1
        meth <- rbinom(cg.r.num, 1, theta1)
        reads <- rbind(reads, c(i, sum(meth), cg.r.num)) 
      }
    } 
  } else {
    for(c in 1:cpg.num){
      if(runif(1) >= missing.rate) {
        cg.r.num <- rpois(1,read.num)+1
        meth <- rbinom(cg.r.num, 1, theta2)
        reads <- rbind(reads, c(i, sum(meth), cg.r.num)) 
      }
    }
  }
  
  reads
}

############################################## module 1
setwd("/path/to/working/directory")
source("beta_mixture_model.r")

# To produce simulation data by 10000 sampling
mat <- c()
for(i in 1:10000) {
  rd <- runif(2)
  theta1 <- max(rd) 
  theta2 <- min(rd)
  
  lambda <- sample(seq(0.1,1,by=0.1),1)
  td <- c(); ct <- c(0); ld <- c()
  for(j in 1:100) {
    # to call simulation function
    reads <- simul.data(cpg.num=10, read.num=30, ncells=100, missing.rate=0.0, lambda=lambda, theta1=theta1, theta2=theta2)
    
    # to format data as the input of beta_mixture_model
    matList <- NULL
    for(i in unique(reads[,1])) {
      if(is.null(matList)) matList <- list(reads[reads[,1]==i,2:3]) else matList <- c(matList, list(reads[reads[,1]==i,2:3]))
    }
    # to call beta_mixture_model
    out <- beta_mixture_model(matList); 
    # to exclude abnornal simulation
    if(is.null(out)) next 
    
    # to harvest the estimated parameters from simulation data
    ld <- c(ld, out$lambda)
    td <- c(td, abs(out$theta1-out$theta2))
    # to count the accurate estimations when the tolerance of estimation is at most 1e-2
    if(abs(lambda-out$lambda)<1e-2 && abs((theta1-theta2)-abs(out$theta1-out$theta2))<1e-2) ct <- ct + 1
  }
  # to obatin estimators of parameters
  mat <- rbind(mat, c(lambda, theta1-theta2, mean(ld), mean(td), ct/100))
}
# to output simulation results
colnames(mat) <- c("lambda", "real.diff", "esitmated", "estimated.diff", "percison")
write.table(mat, file="simul.dat1.txt", quote=F, row.names=F, sep="\t")
### run module 1 again to produce "simul.dat2.txt"

############################################## module 2
# two simulations to obtain 10000 sampling in total; for each of simulation, it takes almost 5 hours. 
x <- read.table('simul.dat1.txt', h=T)
y <- read.table('simul.dat2.txt', h=T)

# to plot |theta1-theta2| vs. percison for different lambda 
dat <- rbind(x[x[,5]>0.05 & x[,1]!=1,], y[y[,5]>0.05 & y[,1]!=1,])
library(ggplot2)
windowsFonts(Times=windowsFont("Times New Roman"))
df <- data.frame(lambda=factor(dat[,1]), real.diff=factor(round(10*dat[,2])/10), percison=as.double(dat[,5]))
tiff('Percision.simul.data.tiff', width=800, height=600)
ggplot(df, aes(x = real.diff, y = percison, fill = lambda)) + geom_boxplot(outlier.color="white", width=0.8) + ylim(0,1) + 
labs(title="", x=expression(bold(theta^(1)-theta^(2))), y = "Fraction of accurate prediction", fill=expression(bold(lambda))) + theme_classic() + 
theme(axis.text.x = element_text(family="Times",face="bold",size=20, color = "black"), axis.text.y = element_text(family="Times",face="bold",size=20, color = "black")) + 
theme(axis.title.y = element_text(family="Times",face="bold",size=20), axis.title.x = element_text(family="Times",face="bold",size=20)) + 
theme(legend.text = element_text(family="Times",face="bold",size = 20), legend.title = element_text(family="Times",face="bold",size=20)) + 
theme(legend.position="right", legend.key.size = unit(1, "cm"))
dev.off()
# to get all numeric output
output <- c()
for(j in levels(df$real.diff)) for(i in levels(df$lambda)) {
  temp <- df[grepl(i,df$lambda) & grepl(j,df$real.diff),"percison"]
  output <- rbind(output, data.frame(lambda=i, diff=j, median=median(temp), std=sd(temp)))
}
# > output
# num  lambda diff median.percison std.percison
# 10    0.1  0.1  0.275 0.20956343
# 11    0.2  0.1  0.330 0.22958009
# 12    0.3  0.1  0.350 0.22385647
# 13    0.4  0.1  0.330 0.26077863
# 14    0.5  0.1  0.370 0.24918357
# 15    0.6  0.1  0.395 0.25286345
# 16    0.7  0.1  0.335 0.26913974
# 17    0.8  0.1  0.530 0.24280460
# 18    0.9  0.1  0.380 0.21277146
# 19    0.1  0.2  0.700 0.19442705
# 20    0.2  0.2  0.825 0.24544689
# 21    0.3  0.2  0.890 0.22631094
# 22    0.4  0.2  0.900 0.23577042
# 23    0.5  0.2  0.920 0.27785226
# 24    0.6  0.2  0.900 0.27237165
# 25    0.7  0.2  0.890 0.23492965
# 26    0.8  0.2  0.835 0.25409416
# 27    0.9  0.2  0.710 0.18849824
# 28    0.1  0.3  0.730 0.08709891
# 29    0.2  0.3  0.860 0.12771038
# 30    0.3  0.3  0.900 0.17217227
# 31    0.4  0.3  0.930 0.11981871
# 32    0.5  0.3  0.930 0.18934039
# 33    0.6  0.3  0.920 0.22614521
# 34    0.7  0.3  0.910 0.15963962
# 35    0.8  0.3  0.860 0.22654565
# 36    0.9  0.3  0.740 0.18726030
# 37    0.1  0.4  0.730 0.13457813
# 38    0.2  0.4  0.860 0.13962403
# 39    0.3  0.4  0.900 0.16269606
# 40    0.4  0.4  0.920 0.21544456
# 41    0.5  0.4  0.930 0.10547051
# 42    0.6  0.4  0.930 0.09088004
# 43    0.7  0.4  0.920 0.11286123
# 44    0.8  0.4  0.870 0.12342616
# 45    0.9  0.4  0.760 0.17576747
# 46    0.1  0.5  0.775 0.11139776
# 47    0.2  0.5  0.870 0.11203515
# 48    0.3  0.5  0.910 0.05799475
# 49    0.4  0.5  0.925 0.10367011
# 50    0.5  0.5  0.940 0.09287071
# 51    0.6  0.5  0.930 0.08482417
# 52    0.7  0.5  0.910 0.05614256
# 53    0.8  0.5  0.870 0.09593278
# 54    0.9  0.5  0.770 0.14126599
# 55    0.1  0.6  0.765 0.09049279
# 56    0.2  0.6  0.870 0.05404871
# 57    0.3  0.6  0.920 0.03781100
# 58    0.4  0.6  0.930 0.03289389
# 59    0.5  0.6  0.930 0.02784421
# 60    0.6  0.6  0.930 0.03043708
# 61    0.7  0.6  0.910 0.03908964
# 62    0.8  0.6  0.880 0.05067036
# 63    0.9  0.6  0.770 0.09719712
# 64    0.1  0.7  0.800 0.07654729
# 65    0.2  0.7  0.900 0.04528470
# 66    0.3  0.7  0.910 0.02941814
# 67    0.4  0.7  0.930 0.02694085
# 68    0.5  0.7  0.940 0.02178477
# 69    0.6  0.7  0.930 0.02836680
# 70    0.7  0.7  0.910 0.03533378
# 71    0.8  0.7  0.865 0.05103687
# 72    0.9  0.7  0.800 0.09244005
# 73    0.1  0.8  0.820 0.06320794
# 74    0.2  0.8  0.890 0.04652718
# 75    0.3  0.8  0.940 0.03317385
# 76    0.4  0.8  0.945 0.02824496
# 77    0.5  0.8  0.960 0.02185092
# 78    0.6  0.8  0.950 0.02524852
# 79    0.7  0.8  0.930 0.03884403
# 80    0.8  0.8  0.915 0.04884436
# 81    0.9  0.8  0.820 0.07090359
# 82    0.1  0.9  0.900 0.06280923
# 83    0.2  0.9  0.980 0.03183473
# 84    0.3  0.9  0.965 0.02796473
# 85    0.4  0.9  0.970 0.01829095
# 86    0.5  0.9  0.980 0.01804693
# 87    0.6  0.9  0.970 0.01959106
# 88    0.7  0.9  0.980 0.02619477
# 89    0.8  0.9  0.960 0.02592410
# 90    0.9  0.9  0.915 0.06199579

# to produce a table of lambda estimation
df <- data.frame(lambda=factor(dat[,1]), elambda=as.double(dat[,3]))
tab <- c()
for(i in levels(df$lambda)) {
  temp <- df[grepl(i, df[,1]),2]
  quant <- as.double(quantile(temp, c(0.25, 0.75)))
  tab <- rbind(tab, data.frame(lambda=i, median=median(temp), mean=mean(temp), std=sd(temp), Q.0.25=quant[1], Q.0.75=quant[2]))
}
# > tab
# num  lambda median  mean     std      Q.0.25 Q.0.75
# 1    0.1    0.1 0.1262259 0.11931794    0.1    0.1
# 2    0.2    0.2 0.2174693 0.10834583    0.2    0.2
# 3    0.3    0.3 0.3140649 0.09208669    0.3    0.3
# 4    0.4    0.4 0.4128731 0.10036005    0.4    0.4
# 5    0.5    0.5 0.4994332 0.09038868    0.5    0.5
# 6    0.6    0.6 0.5957247 0.09057940    0.6    0.6
# 7    0.7    0.7 0.6979118 0.07308922    0.7    0.7
# 8    0.8    0.8 0.7947195 0.08307829    0.8    0.8
# 9    0.9    0.9 0.8860523 0.09225335    0.9    0.9

# difference of thetas vs. difference of thetas estimated 
df <- data.frame(lambda=factor(dat[,1]), real.diff=as.double(dat[,2]), estimated.diff=as.double(dat[,4]))
cols <- rainbow(length(levels(df$lambda)))
tiff('Theta.diff.simul.data.tiff', width=1500, height=1500)
par(mfrow=c(3,3), mai=c(1,1,1,1),family="Times")
for(i in levels(df$lambda)) {
  makeTransparent = function(..., alpha=0.5) {
    if(alpha<0 | alpha>1) stop("alpha must be between 0 and 1")
    alpha = floor(255*alpha)  
    newColor = col2rgb(col=unlist(list(...)), alpha=FALSE)

    .makeTransparent = function(col, alpha) {
      rgb(red=col[1], green=col[2], blue=col[3], alpha=alpha, maxColorValue=255)
    }

    newColor = apply(newColor, 2, .makeTransparent, alpha=alpha)
    return(newColor)
  }
  
  plot(NA, xlab="", ylab="", main="", cex.axis=3, cex.lab=3, cex.main=3, font.axis=2, font.lab=2, xlim=c(0,1), ylim=c(0,1))
  temp <- df[grepl(i,df$lambda),2:3]
  temp <- temp[order(temp[,1]),]
  z <- smooth.spline(temp[,1], temp[,2], df=200)
  points(z$x, z$y, col=cols[which(grepl(i, levels(df$lambda)))], cex=1, pch=16)
  
  vect <- c()
  for(j in seq(0.05,0.95, by=0.05)) {
    tmp <- temp[temp[,1]>=j-0.02 & temp[,1]<j+0.02,2]
    vect <- rbind(vect, c(j, mean(tmp)-sd(tmp), mean(tmp)+sd(tmp)))
  }
  lower <- smooth.spline(vect[,1], vect[,2], df=5)
  upper <- smooth.spline(vect[,1], vect[,3], df=5)
  polygon(c(lower$x, rev(upper$x)), c(lower$y, rev(upper$y)), col=makeTransparent("gray", alpha=0.5))
  
  mtext(bquote(bold("\U03BB" ~ "=" ~ .(i))), side=3, line=2, cex=3, font=2)
  mtext(bquote(bold("\U03B8"^"(1)"-"\U03B8"^"(2)")), side=1, line=6, cex=3, font=2)
  mtext(bquote(bold("Estimated " ~ "\U03B8"^"(1)"-"\U03B8"^"(2)")), side=2, line=3.5, cex=3, font=2)
 
  ct <- cor.test(temp[,1], temp[,2])
  text(0.3, 0.8, sprintf("Pearson's R=%.2f,\np-value=%s", ct$estimate, ifelse(ct$p.value>0, format(ct$p.value, scientific=T, digits=3), "0")), cex=3, font=2)
}
dev.off()

